#!/usr/bin/perl
#
#
# SYNTAX:
# 	populate_db.pl [initial | icons | plugins]
#
# DESCRIPTION:
# 	Populates the database based on plugins in libexec
#	If $1 = "initial", then will add in a set of default services and a
#	servicegroup called operations
#	If $1 = "icons", will regenerate icons from hosticons.db file and plugins
#	If $1 = "plugins", will only redo all the plugin helps
#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use strict;
use FindBin qw($Bin);
use lib "/usr/local/nagios/perl/lib";

# Need to set this as plugins expect to be available and an "su - nagios -c populate_db.pl" doesn't set
$ENV{PERL5LIB} = "/usr/local/nagios/perl/lib";

use lib "$Bin/../lib", "$Bin/../etc";
use Opsview::Host;
use Opsview::Timeperiod;
use Data::Dumper;
use Text::CSV::Simple;
use Sys::Hostname;

my $param = shift @ARGV || "plugins";

populate_plugins() unless ( $param eq "initial" );
populate_icons() if ( $param eq "initial" or $param eq "icons" );
populate_initial() if ( $param eq "initial" );

sub populate_plugins {

    my %use_help = ( "check_cluster" => 1, );

    chdir "/usr/local/nagios/libexec";
    my @plugins = <check_*>;
    foreach my $plugin (@plugins) {
        next if ( $plugin =~ /\.dpkg-tmp/ );
        next if ( $plugin =~ /\.tmp/ );
        next if ( $plugin =~ /\.bak/ );
        my $command = "./$plugin -h";
        if ( $use_help{$plugin} ) {
            $command = "./$plugin --help";
        }
        my @output = `$command`;
        my $obj = Opsview::Plugin->find_or_create( { name => $plugin }, { onserver => 1, help => join( "", @output ) } );
    }
}

sub populate_icons {
    system( $Bin. '/hosticon_admin', 'import', $Bin . '/../import/hosticons.db' ) == 0 || die( 'Cannot import hosticons.db', $/ );
}

sub populate_initial {

    # This is a no-op as it is now handled via the import/initial_opsview.sql file
}

# Keep this logic here, as we will have comma separated files to do mass imports in future
sub populate_from_file {
    ##################################################################
    # These are tasks that only need to be done on an initial population
    ##################################################################

    my $allhours = Opsview::Timeperiod->find_or_create(
        {   name      => "24x7",
            alias     => "24 Hours A Day, 7 Days A Week",
            sunday    => "00:00-24:00",
            monday    => "00:00-24:00",
            tuesday   => "00:00-24:00",
            wednesday => "00:00-24:00",
            thursday  => "00:00-24:00",
            friday    => "00:00-24:00",
            saturday  => "00:00-24:00",
        }
    );
    my $sgs    = {};
    my $parser = Text::CSV::Simple->new;

    open SERVICES, "$Bin/../import/servicechecks.db" or die "Cannot open servicechecks.db";
    my $head = <SERVICES>;
    close SERVICES;
    chop($head);
    $head =~ s/^#//;
    my @attrs = split( ",", $head );
    $parser->field_map(@attrs);
    my @data = $parser->read_file("$Bin/../import/servicechecks.db");
    shift @data;    # 1st line denotes all the fieldnames

    foreach my $data (@data) {
        unless ( $sgs->{ $data->{servicegroup} } ) {
            $sgs->{ $data->{servicegroup} } = Opsview::Servicegroup->find_or_create( { name => $data->{servicegroup} } );
        }
        $data->{servicegroup} = $sgs->{ $data->{servicegroup} };
        my $plugin = Opsview::Plugin->search( name => $data->{plugin} );
        unless ($plugin) {
            print "Plugin $data->{plugin} not found, ignoring servicecheck $data->{name}", $/;
            next;
        }

        #print Dumper $data;
        my $name = delete $data->{name};
        $data->{check_period} = $allhours->id;
        Opsview::Servicecheck->find_or_create( { name => $name }, $data );
    }
    close SERVICES;
}
