#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

package Opsview::ResultSet::MonitorPacks;

use strict;
use warnings;

use base qw/Opsview::ResultSet/;
use Opsview::Utils qw(convert_to_arrayref);

use Log::Log4perl;
use version;
use JSON qw(decode_json);
use Try::Tiny;
use File::Copy;
use File::Path;

sub monitor_pack_dir {
    my $dir = "/usr/local/nagios/var/spool/monitorpacks";
    if ( !-e $dir ) {
        mkdir $dir || die "Cannot create $dir";
    }
    $dir;
}

# Assume this is invoked from the monitorpacks directory
# Will die if failures found
sub install_pack {
    my ( $self, $filename, $opts ) = @_;
    $opts ||= {};

    my $plugindir = "/usr/local/nagios/libexec";

    my $force = $opts->{force};

    my $logger = $opts->{logger};
    unless ($logger) {
        Log::Log4perl::init("/usr/local/nagios/etc/Log4perl.conf");
        $logger = Log::Log4perl->get_logger("monitorpacks");
    }
    if ( $opts->{stdout} ) {
        my $stdout_appender = Log::Log4perl::Appender->new(
            "Log::Log4perl::Appender::Screen",
            name   => "screenlog",
            stderr => 0,
        );
        $logger->add_appender($stdout_appender);
    }

    my $packdir = $filename;
    $packdir =~ s/\.tar\.gz$//;

    if ( -e $packdir ) {
        $logger->info("Found older version of $packdir - deleting");
        rmtree($packdir);
    }

    $logger->debug("Uncompressing $filename");
    my $rc = system( "tar", "--gzip", "-xf", $filename );
    if ( $rc != 0 ) {
        $logger->logdie("Cannot uncompress $filename: rc=$?");
    }

    if ( !-e $packdir ) {
        $logger->logdie("$packdir not found - invalid monitoring pack");
    }

    # Need to touch the top level dir so that will not re-untar the file next time round
    utime time(), time(), $packdir;

    open F, "$packdir/info" or $logger->logdie("Cannot open $packdir/info");
    my $info = {};
    while (<F>) {
        next if /^#/;
        if (/^([A-Z_]+)=(.*)$/) {
            my $k = lc $1;
            $info->{$k} = $2;
        }
    }
    close F;

    foreach my $k (qw(name version alias)) {
        $logger->logdie("No $k in $packdir/info") unless $info->{$k};
    }

    my $mpid = "Monitor pack '" . $info->{alias} . "'";

    # Check Opsview minimum version
    if ( $info->{opsview_min_version} ) {
        if ( version->parse( Opsview::Config->opsview_version ) <= version->parse( $info->{opsview_min_version} ) ) {
            $logger->logdie( "Error: Minimum Opsview version for monitor pack '" . $info->{alias} . "' is " . $info->{opsview_min_version} . " but currently on " . Opsview::Config->opsview_version );
        }
    }

    my $mpack = $self->find_or_new( { name => $info->{name} } );
    unless ( $mpack->in_storage ) {
        $mpack->insert;
    }
    elsif ( !$force ) {
        if ( $mpack->status ne "OK" ) {
            $logger->logdie( "Error: $mpid already installed in state " . $mpack->status );
        }
        if ( version->parse( $mpack->version ) == version->parse( $info->{version} ) ) {
            $logger->logdie("Error: $mpid - version $info->{version} is already installed");
        }
        elsif ( version->parse( $mpack->version ) > version->parse( $info->{version} ) ) {
            $logger->logdie( "Error: $mpid installed version " . $mpack->version . " is greater than " . $info->{version} );
        }
    }
    $mpack->update( { alias => $info->{alias}, version => $info->{version}, dependencies => $info->{dependencies} } );
    $mpack->update( { status => "INSTALLING" } );

    # Check dependencies - TODO
    my @deps = $self->parse_dependencies( $info->{dependencies} );

    # Check plugins if already exists
    my @errors;
    opendir DIR, "$packdir/plugins" or $logger->logdie("Cannot open $packdir/plugins/ dir");
    my @plugins = grep !/^\.\.?/, readdir DIR;
    closedir DIR;

    my $plugins_rs = $self->result_source->schema->resultset("Plugins");
    my @plugins_to_add_to_db;
    foreach my $p (@plugins) {
        if ( $plugins_rs->find( { name => $p } ) ) {
            push @errors, "Plugin '$p' already exists";
        }
        else {
            push @plugins_to_add_to_db, $p;
        }
    }

    # Read config.json
    open F, "$packdir/config.json" or $logger->logdie("Cannot open $packdir/config.json");
    my $config_perl;
    {
        local $/;
        my $json_text = <F>;
        $config_perl = decode_json($json_text);
    }
    close F;

    # Check service groups
    foreach my $sg ( @{ $config_perl->{servicegroup} } ) {
        if ( $self->result_source->schema->resultset("Servicegroups")->find( { name => $sg->{name} } ) ) {
            push @errors, "Service group '" . $sg->{name} . "' already exists";
        }
    }

    # Check attributes
    foreach my $at ( @{ $config_perl->{attribute} } ) {
        if ( $self->result_source->schema->resultset("Attributes")->find( { name => $at->{name} } ) ) {
            push @errors, "Attribute '" . $at->{name} . "' already exists";
        }
    }

    # Check service checks
    foreach my $sc ( @{ $config_perl->{servicecheck} } ) {
        if ( $self->result_source->schema->resultset("Servicechecks")->find( { name => $sc->{name} } ) ) {
            push @errors, "Service check '" . $sc->{name} . "' already exists";
        }
    }

    # Check host templates
    foreach my $ht ( @{ $config_perl->{hosttemplate} } ) {
        if ( $self->result_source->schema->resultset("Hosttemplates")->find( { name => $ht->{name} } ) ) {
            push @errors, "Host template '" . $ht->{name} . "' already exists";
        }
    }

    # If errors and ! force, log and return
    if (@errors) {
        if ($force) {
            my $message = "$mpid - errors found, but continuing with import: " . join( ", ", @errors );
            $logger->info($message);
        }
        else {
            my $message = "$mpid - errors found. No import attempted: " . join( ", ", @errors );
            $mpack->update( { message => $message, status => "FAILURE" } );
            $logger->logdie($message);
        }
    }

    # Make DB changes
    my $error_message;
    try {
        $logger->info("Importing");
        my $guard = $self->result_source->schema->txn_scope_guard;

        # Update plugins table first, then copy plugins for opsviewd to scan
        foreach my $p (@plugins_to_add_to_db) {
            $self->result_source->schema->resultset("Plugins")->update_or_create( { name => $p } );
        }
        foreach my $p (@plugins) {
            $logger->info("Adding plugin: $p");
            copy( "$packdir/plugins/$p", $plugindir ) or $logger->error("Cannot copy $p to $plugindir");
        }

        my $sync_opts = {};
        unless ($force) {
            $sync_opts->{create_only} = 1;
        }

        # Update service groups
        foreach my $sg ( @{ $config_perl->{servicegroup} } ) {
            $logger->info( "Adding servicegroup: " . $sg->{name} );
            $self->result_source->schema->resultset("Servicegroups")->synchronise( $sg, $sync_opts );
        }

        # Update attributes
        foreach my $at ( @{ $config_perl->{attribute} } ) {
            $logger->info( "Adding attribute: " . $at->{name} );
            $self->result_source->schema->resultset("Attributes")->synchronise( $at, $sync_opts );
        }

        # Update service checks
        foreach my $sc ( @{ $config_perl->{servicecheck} } ) {
            $logger->info( "Adding service check: " . $sc->{name} );
            $self->result_source->schema->resultset("Servicechecks")->synchronise( $sc, $sync_opts );
        }

        # Update host templates
        foreach my $ht ( @{ $config_perl->{hosttemplate} } ) {
            $logger->info( "Adding host template: " . $ht->{name} );
            $self->result_source->schema->resultset("Hosttemplates")->synchronise( $ht, $sync_opts );
        }

        $guard->commit;
    }
    catch {
        $error_message = $_;
    };

    # Update $mpack with result
    if ( defined $error_message ) {
        $mpack->update( { status => "FAILED", message => $error_message } );
        $logger->logdie("$mpid - database update failed: $error_message");
    }

    $mpack->update( { status => "OK", message => "" } );
    $logger->info("$mpid - finished installation");
}

sub parse_dependencies {
    my ( $self, $deps ) = @_;
    return split( /\s+/, $deps );
}

use Cwd;

sub install_new_monitorpacks {
    my ( $self, $logger ) = @_;

    # For each tarball, check if the timestamp is newer than the expanded directory
    my $monitorpackdir = $self->monitor_pack_dir;
    my $cwd            = getcwd;
    chdir($monitorpackdir) or $logger->logdie("Cannot chdir to $monitorpackdir");

    opendir DIR, ".";
    my %files = map { ( $_ => 1 ) } grep !/^\.\.?/, readdir DIR;
    closedir DIR;

    my @monitorpacks = grep /\.tar\.gz$/, keys %files;
    foreach my $mp (@monitorpacks) {
        ( my $mpname = $mp ) =~ s/\.tar\.gz//;
        if ( exists $files{$mpname} && ( ( stat($mpname) )[9] >= ( stat($mp) )[9] ) ) {
            next;
        }

        # Run install_monitorpack
        try {
            $self->install_pack( $mp, { logger => $logger } );
        }
    }

    chdir($cwd);
}

1;
