#
#
# AUTHORS:
#	Copyright (C) 2003-2011 Opsera Limited. All rights reserved
#
#    This file is part of Opsview
#
#    Opsview is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Opsview is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Opsview; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

package Reports::Report;
use warnings;
use strict;
use base "Reports";

__PACKAGE__->table("reports");

__PACKAGE__->columns( All => qw/id name selection type format logo scheduled email/, );

__PACKAGE__->default_search_attributes( { order_by => "name" } );

__PACKAGE__->has_a( selection => "Reports::Selection" );
__PACKAGE__->has_a( type      => "Reports::ReportType" );

__PACKAGE__->initial_columns(qw(name type));

__PACKAGE__->constrain_column_regexp( name => q{/^[\w \.\/\+-]+$/} => q{Report name can only consist of alphanumerics, space or the following characters: -./+} );

__PACKAGE__->might_have( weekly_availability_keyword_critical_states => "Reports::Report::WeeklyAvailabilityKeywordCriticalStates" => qw/start_of_week label keywords annotation auto_null_annotation/ );

=head1 NAME

Reports::ReportType - Metadata about a report type. Not user configurable - entries added with new report types

=head1 METHODS

=over 4

=item $self->actionname

Returns the action name

=cut

sub actionname {
    my ($self) = @_;
    if ( ( $_ = $self->type->fixedname ) eq "generic_availability" ) {
        return;
    }
    else {
        s/_//g;
        return $_;
    }
}

=item $self->ready_to_auto_generate_today

Returns true if today is the right day to auto generate the report

=cut

use DateTime;

sub ready_to_auto_generate_today {
    my $self = shift;
    my $dt = DateTime->now( time_zone => "local" );
    if ( $self->type->frequency eq "monthly" ) {
        my $day_in_month = $dt->day;
        if ( $day_in_month == $self->type->day_value ) {
            return 1;
        }
        else {
            return 0;
        }
    }
    elsif ( $self->type->frequency eq "weekly" ) {

        # Mod 7 because Monday = 1 and goes from 1-7
        my $day_in_week = $dt->day_of_week % 7;

        # TODO: This should use $self->type->day_value, but not done yet
        if ( $day_in_week == $self->start_of_week ) {
            return 1;
        }
        else {
            return 0;
        }
    }
}

=item $self->previous_reporting_period

Returns a YYYY-MM-DD format for the prior reporting date, based on today and the frequency of the report

=cut

sub previous_reporting_period {
    my $self = shift;
    my $dt = DateTime->now( time_zone => "local" );
    if ( $self->type->frequency eq "monthly" ) {
        $dt->subtract( months => 1 );
    }
    elsif ( $self->type->frequency eq "weekly" ) {
        $dt->subtract( weeks => 1 );
    }
    return $dt->ymd;
}

# TODO: This should be table based

=item $self->capability( $capability )

Returns true if this report can do $capability. Current list: annotations

=cut

sub capability {
    my ( $self, $cap ) = @_;
    if ( $cap eq "annotations" && $self->type->fixedname eq "weekly_availability_keyword_critical_states" ) {
        return 1;
    }
    return 0;
}

=item $self->custom($selection)

Gets custom information. API incomplete

=cut

sub custom {
    my ( $self, $selection ) = @_;
    my %hostgroups = map { ( $_->hostgroup => { hostgroup => $_->hostgroup, warning => "", critical => "" } ) } ( $selection->hostgroups );
    my $sth = $self->db_Main->prepare_cached("SELECT * FROM generic_availability_targets_hostgroup");
    $sth->execute;
    my @list;
    while ( my $row = $sth->fetchrow_hashref ) {
        if ( $row->{hostgroup} eq "" ) {
            push @list, { hostgroup => "", warning => $row->{warning}, critical => $row->{critical} };
        }
        if ( exists $hostgroups{ $row->{hostgroup} } ) {
            $hostgroups{ $row->{hostgroup} }->{warning}  = $row->{warning};
            $hostgroups{ $row->{hostgroup} }->{critical} = $row->{critical};
        }
    }
    foreach my $hg ( sort keys %hostgroups ) {
        push @list, $hostgroups{$hg};
    }
    return \@list;
}

=item set_hostgroups_on_off

Sets warning and critical levels for hostgroups

=cut

sub set_hostgroups_on_off {
    my $self = shift;
    my ( $all, $on, $warning, $critical ) = @_;
    my %active;
    map { $active{$_}++ } @$on;

    foreach $_ (@$on) {
        if ( $_ eq "Default" || $_ eq "" ) { $_ = "" }
        my $obj      = Reports::GenericAvailabilityTargetsHostgroup->search( { hostgroup => $_, } )->first();
        my $warning  = shift @$warning;
        my $critical = shift @$critical;
        $warning  = undef if $warning  eq "default";
        $critical = undef if $critical eq "default";
        if ($obj) {
            if ( !defined $warning && !defined $critical ) {
                $obj->delete;
            }
            else {
                $obj->warning($warning);
                $obj->critical($critical);
                $obj->update;
            }
        }
        else {
            if ( defined $warning || defined $critical ) {
                my $args = { hostgroup => $_, warning => $warning, critical => $critical };
                $obj = Reports::GenericAvailabilityTargetsHostgroup->create($args);
            }
        }
    }
}

=item my_type_is

Returns "scheduled report"

=cut

sub my_type_is {
    return "scheduled report";
}

=back

=head1 AUTHOR

Opsera Limited

=head1 LICENSE

GNU General Public License v2

=cut

1;
